package org.xml.io;

import java.io.Writer;
import java.io.FilterWriter;
import java.io.IOException;
import java.util.Hashtable;

public class XMLWriter extends FilterWriter {
  private String padding;

  private int depth;
  private Entity[] entities;

  public XMLWriter(Writer out, String padding) {
    super(out);
    this.padding = padding;
    depth = 0;
  }

  public void writeDocument(Element rootElement, String ID) throws IOException {
    writeDocument(rootElement, ID, null, true);
  }

  public void writeDocument(Element rootElement, String ID, Entity[] entities) throws IOException {
    writeDocument(rootElement, ID, entities, true);
  }

  public void writeDocument(Element rootElement, String ID, Entity[] entities, boolean system) throws IOException {
    this.entities = entities;

    out.write("<!DOCTYPE ");
    out.write(rootElement.getName());
    out.write((system) ? " SYSTEM \"" : " PUBLIC \"");
    out.write(ID);
    out.write("\">\n");

    writeElement(rootElement);
  }

  // recursively write out elements and their children
  public void writeElement(Element element) throws IOException {
    int i;
    for (i = 0; i < depth; i++) {
      out.write(padding);
    }

    depth++;

    if (element.getComments() != null) {
      out.write(element.getComments());
      out.write('\n');
    }
    out.write('<');
    out.write(element.getName());
    Attribute[] attributes = element.getAttributes();
    if (attributes != null) {
      for (i = 0; i < attributes.length; i++) {
        out.write(' ');
        out.write(attributes[i].getName());
        out.write('=');
        out.write(attributes[i].getValue());
      }
    }
    out.write('>');

    Element[] children = element.getChildren();
    if (children != null && children.length > 0) {
      out.write('\n');
      for (i = 0; i < children.length; i++) {
        writeElement(children[i]);
      }
    }
    else if (element.getContent() != null) {
      out.write((entities != null) ? replaceText(element.getContent()) : element.getContent());
    }

    out.write('<');
    out.write('/');
    out.write(element.getName());
    out.write('>');
    out.write('\n');

    depth--;
  }

  private String replaceText(String content) {
    String replaceText = content;

    StringBuffer currentText;
    String value;
    int e, i, l;
    for (e = 0; e < entities.length; e++) {
      value = entities[e].getValue();
      currentText = new StringBuffer();
      i = l = 0;
      while ((i = replaceText.indexOf(value, l)) != -1) {
        currentText.append(content.substring(0, i));
        currentText.append('&');
        currentText.append(entities[e].getName());
        currentText.append(';');
        l = i + value.length();
      }
      currentText.append(replaceText.substring(l));
      replaceText = currentText.toString();
    }
    return replaceText;
  }
}
